@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1993-2001 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1993-2005 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -20,7 +20,13 @@ struct screenhack {
   char *command;
 };
 
-typedef enum { RANDOM_HACKS, ONE_HACK, BLANK_ONLY, DONT_BLANK } saver_mode;
+typedef enum {
+  RANDOM_HACKS, ONE_HACK, BLANK_ONLY, DONT_BLANK, RANDOM_HACKS_SAME
+} saver_mode;
+
+typedef enum {
+  TEXT_DATE, TEXT_LITERAL, TEXT_FILE, TEXT_PROGRAM, TEXT_URL
+} text_mode;
 
 typedef struct saver_preferences saver_preferences;
 
@@ -41,6 +47,8 @@ struct saver_preferences {
   Bool verbose_p;		/* whether to print out lots of status info */
   Bool timestamp_p;		/* whether to mark messages with a timestamp */
   Bool capture_stderr_p;	/* whether to redirect stdout/stderr  */
+  Bool ignore_uninstalled_p;	/* whether to avoid displaying or complaining
+                                   about hacks that are not on $PATH */
   Bool debug_p;			/* pay no mind to the man behind the curtain */
   Bool xsync_p;			/* whether XSynchronize has been called */
 
@@ -56,6 +64,8 @@ struct saver_preferences {
   Bool install_cmap_p;		/* whether we should use our own colormap
 				   when using the screen's default visual. */
 
+  Bool quad_p;			/* whether to run four savers per monitor */
+
   screenhack **screenhacks;	/* the programs to run */
   int screenhacks_count;
 
@@ -74,6 +84,7 @@ struct saver_preferences {
   Time pointer_timeout;		/* how often to check mouse position */
   Time notice_events_timeout;	/* how long after window creation to select */
   Time watchdog_timeout;	/* how often to re-raise and re-blank screen */
+  int pointer_hysteresis;	/* mouse motions less than N/sec are ignored */
 
   Bool dpms_enabled_p;		/* Whether to power down the monitor */
   Time dpms_standby;		/* how long until monitor goes black */
@@ -85,17 +96,26 @@ struct saver_preferences {
   Bool random_image_p;		/*  "xscreensaver-getimage" program, and set */
   char *image_directory;	/*  by the "xscreensaver-demo" configurator. */
 
+  text_mode tmode;		/* How we generate text to display. */
+  char *text_literal;		/* used when tmode is TEXT_LITERAL. */
+  char *text_file;		/* used when tmode is TEXT_FILE.    */
+  char *text_program;		/* used when tmode is TEXT_PROGRAM. */
+  char *text_url;		/* used when tmode is TEXT_URL.     */
+
   Bool use_xidle_extension;	/* which extension to use, if possible */
   Bool use_mit_saver_extension;
   Bool use_sgi_saver_extension;
   Bool use_proc_interrupts;
 
+  Bool getviewport_full_of_lies_p; /* XFree86 bug #421 */
+
   char *shell;			/* where to find /bin/sh */
 
   char *demo_command;		/* How to enter demo mode. */
   char *prefs_command;		/* How to edit preferences. */
   char *help_url;		/* Where the help document resides. */
   char *load_url_command;	/* How one loads URLs. */
+  char *new_login_command;	/* Command for the "New Login" button. */
 };
 
 
