@@ -19,19 +19,16 @@
 #endif
 
 #include <GL/gl.h>
-#include <GL/glut.h>
+/*#include <GL/glut.h>*/
 #ifdef HAVE_GLE3
 #include <GL/gle.h>
 #else
 #include <GL/tube.h>
 #endif
 
-/*  most recent mouse postion */
+/* controls shape of object */
 extern float lastx;
 extern float lasty;
-extern float rot_x;
-extern float rot_y;
-extern float rot_z;
 
 void InitStuff_helix3 (void) 
 {
@@ -45,12 +42,9 @@ void DrawStuff_helix3 (void)
 
    /* set up some matrices so that the object spins with the mouse */
    glPushMatrix ();
-   glTranslatef (0.0, 0.0, -80.0);
-   glRotatef(rot_x, 1, 0, 0);
-   glRotatef(rot_y, 0, 1, 0);
-   glRotatef(rot_z, 0, 0, 1);
-/*     glRotatef (220.0, 0.0, 1.0, 0.0); */
-/*     glRotatef (65.0, 1.0, 0.0, 0.0); */
+/* glTranslatef (0.0, 0.0, -80.0); */
+/* glRotatef (220.0, 0.0, 1.0, 0.0); */
+/* glRotatef (65.0, 1.0, 0.0, 0.0); */
 
    /* Phew. FINALLY, Draw the helix  -- */
    gleSetJoinStyle (TUBE_NORM_EDGE | TUBE_JN_ANGLE | TUBE_JN_CAP);
